# -*- coding: utf-8 -*-
#
# gPodder - A media aggregator and podcast client
# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
#
# gPodder is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# gPodder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gpodder

_ = gpodder.gettext

HINT_STRINGS = (
    _("Sign up for gpodder.net to synchronize (and backup) your subscriptions to the cloud."),
    _("Bulk-deleting episodes is possible from the main window menu."),
    _("To save power, automatic updates are only enabled while gPodder is running."),
    _("Subscribe to YouTube users and download their videos to your device."),
    _("Use the shortcut URL yt:<username> to quickly subscribe to a YouTube user."),
    _("Search for your fields of interest in the gpodder.net directory to find interesting content."),
    _("Longpress on an episode to access a menu with useful episode-related functions."),
    _("Longpress on an episode in 'New episodes available' to display its shownotes."),
    _("Use the hardware keyboard to filter down lists - simply start typing in the podcast or episode list."),
    _("The top row of the menu in each list window lets you set the filter for it."),
    _("Use fb:<name> as a shorthand for Feedburner URLs (feeds.feedburner.com/<name>)"),
    _("Episodes marked as 'Keep episode' will never be displayed in the 'Delete episodes' list."),
    _("Use gpodder.net to share your subscription list with friends via a private or public URL."),
    _("Found a problem? Report it at bugs.maemo.org (Extras / gPodder), so we can fix it in the next release."),
    _("Got a feature request? Let us know at bugs.maemo.org (Extras / gPodder)!"),
    _("Want to support gPodder? Use the 'donate' button in the about dialog."),
    _("Use Media Player as your audio and video player for position tracking support."),
    _("The 'Delete episodes' dialog has some useful presets in its menu - check them out!"),
    _("Use search-as-you-type in the 'All episodes' list to quickly find a specific episode."),
)


